package com.diagbot.facade;

import com.diagbot.biz.push.entity.Item;
import com.diagbot.biz.push.entity.Lis;
import com.diagbot.biz.push.entity.Pacs;
import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.process.BillProcess;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.BillNeoVO;
import com.diagbot.vo.Drug;
import com.diagbot.vo.IndicationPushVO;
import com.diagbot.vo.ItemExt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @Description: 开单合理性facade
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class BillFacade {

    @Autowired
    NeoFacade neoFacade;
    @Autowired
    BillProcess billProcess;
    @Autowired
    CommonFacade commonFacade;

    /**
     * 开单合理性业务
     * @param
     * @return
     */
    public void billFac(IndicationPushVO indicationPushVO, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 开单合理性图谱接口调用
        BillNeoVO billNeoVO = fillBillNeo(wordCrfDTO);
        List<BillNeoDTO> billNeoDTOList = neoFacade.getBillNeo(billNeoVO);

        // 规则处理
        billProcess.process(billNeoDTOList, wordCrfDTO, res);

        // 结果去重处理
        dealMsg(res);
    }


    /**
     * 提示信息去重
     * @param indicationDTO
     */
    public void dealMsg(IndicationDTO indicationDTO) {
        List<BillMsg> billMsgList = indicationDTO.getBillMsgList();
        if (ListUtil.isNotEmpty(billMsgList)) {
            List<String> msgList = new ArrayList<>();
            Iterator<BillMsg> it = billMsgList.iterator();
            while (it.hasNext()) {
                String msg = it.next().getMsg();
                if (msgList.contains(msg)) {
                    it.remove();
                } else {
                    msgList.add(msg);
                }
            }
        }
    }

    public BillNeoVO fillBillNeo(WordCrfDTO wordCrfDTO) {
        BillNeoVO billNeoVO = new BillNeoVO();

        List<Drug> drugs = wordCrfDTO.getDrugOrder();

        for (Drug item : drugs) {
            billNeoVO.getDrugList().put(item.getName(), item.getUniqueName());
        }

        List<Pacs> pacslist = wordCrfDTO.getPacsOrder();
        for (Pacs pacs : pacslist) {
            billNeoVO.getPacsList().put(pacs.getName(), pacs.getUniqueName());
        }

        List<Lis> lislist = wordCrfDTO.getLisOrder();
        for (Lis lis : lislist) {
            if (StringUtil.isBlank(lis.getDetailName())) {
                billNeoVO.getLisList().put(lis.getName(), lis.getUniqueName());
            }
            else {
                billNeoVO.getLisDetailList().put(lis.getDetailName(), lis.getUniqueName());
            }
        }

        List<Item> operations = wordCrfDTO.getOperationOrder();
        for (Item item : operations) {
            billNeoVO.getOperationList().put(item.getName(), item.getUniqueName());
        }

        List<Item> diags = wordCrfDTO.getDiagOrder();
        for (Item item : diags) {
            billNeoVO.getDiagList().put(item.getName(), item.getUniqueName());
        }

        List<ItemExt> transfusions = wordCrfDTO.getTransfusionOrder();
        for (ItemExt item : transfusions) {
            billNeoVO.getTransfusionList().put(item.getName(), item.getUniqueName());
        }
        return billNeoVO;
    }
}
