package com.diagbot.facade;

import com.alibaba.fastjson.JSONArray;
import com.diagbot.client.ChiefPresentSimilarityServiceClient;
import com.diagbot.client.StandConvertServiceClient;
import com.diagbot.dto.*;
import com.diagbot.entity.node.Symptom;
import com.diagbot.entity.node.YiBaoDiseaseName;
import com.diagbot.enums.StandConvertEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.repository.*;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RedisUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.*;
import com.diagbot.vo.neoPushEntity.ChiefPushVo;
import com.diagbot.vo.neoPushEntity.Diag;
import com.diagbot.vo.neoPushEntity.PresentPushVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Description: 图谱facade
 * @author: Mark
 * @time: 2018/8/6 9:11
 */
@Component
public class NeoFacade {

    @Autowired
    MedicineRepository medicineRepository;
    @Autowired
    PacsNameRepository pacsNameRepository;
    @Autowired
    LisNameRepository lisNameRepository;
    @Autowired
    LisSetRepository lisSetRepository;
    @Autowired
    YiBaoDiseaseRepository yiBaodiseaseRepository;
    @Autowired
    YiBaoOperationNameRepository yiBaoOperationRepository;
    @Autowired
    MedicineCodeRepository medicineCodeRepository;
    @Autowired
    MedJiePouClassRepository medJiePouClassRepository;
    @Autowired
    YiBaoDiseaseRepository yiBaoDiseaseRepository;
    @Autowired
    CombiOperationRepository combiOperationRepository;
    @Autowired
    BaseNodeRepository nodeRepository;
    @Autowired
    ChiefPresentSimilarityServiceClient chiefPresentSimilarityServiceClient;
    @Autowired
    NeoFacade self;
    @Autowired
    StandConvertServiceClient standConvertServiceClient;
    @Autowired
    RedisUtil redisUtil;
    @Value("${StandConvert.rate}")
    String standConvertRate;
    @Autowired
    SymptomNameRepository symptomNameRepository;

    /**
     * 返回药品缓存信息
     *
     * @return
     */
    @Cacheable(value = "cache", key = "'drugCache'")
    public Map<String, List<String>> getDrugCache() {
        Map<String, List<String>> res = new HashMap<>();

        String med_type;
        String med_name;
        String comma="::";

        List<String> medClass = nodeRepository.getMedClass();

        for (String medcls : medClass) {
            String[] spres = medcls.split(comma);
            med_type = spres[1];
            med_name = spres[0];

            if (res.get(med_type) == null) {
                res.put(med_type, new ArrayList<>());
            }
            if (res.get(med_type).indexOf(med_name) == -1) {
                res.get(med_type).add(med_name);
            }
        }


        // TODO 测试数据
        res.put("泰舒达类", Arrays.asList("泰舒达"));
        res.put("青霉素类", Arrays.asList("青霉素"));
        return res;
    }

    /**
     * 图谱反推的数据
     *
     * @param pushVO
     * @return
     */
    public List<NeoPushDTO> getReversePush(NeoPushVO pushVO) {
        List<NeoPushDTO> neoPushDTOS = new ArrayList<>();
        List<String> diags = new ArrayList<>();
        // 如果有诊断名称，则通过诊断反推
        if (null != pushVO.getDiagVo() && pushVO.getDiagVo().getDiags().size() > 0) {
            for (Diag diag : pushVO.getDiagVo().getDiags()) {
                diags.add(diag.getName());
            }

            neoPushDTOS = getDiagInfo(pushVO);
        }

        return neoPushDTOS;
    }


    /**
     * 图谱正推的数据
     *
     * @param pushVO
     * @return
     */
    public List<NeoPushDTO> getPush(NeoPushVO pushVO) {
        List<NeoPushDTO> neoPushDTOS = new ArrayList<>();

        // 如果没有诊断名称，则通过其它信息推送诊断
        ChiefPushVo chiefPushVo = pushVO.getChiefPushVo();
        PresentPushVo presentPushVo = pushVO.getPresentPushVo();
        List<String> symptoms = new ArrayList<>();
        List<String> symptom_chief = new ArrayList<>();
        List<String> symptom_present = new ArrayList<>();
        if(chiefPushVo != null){
            symptom_chief = chiefPushVo.getSymptoms().stream().map(x -> x.getName()).collect(Collectors.toList());
        }
        if(presentPushVo != null){
            symptom_present = presentPushVo.getSymptoms().stream().map(x -> x.getName()).collect(Collectors.toList());
        }
        symptoms = Stream.of(symptom_chief,symptom_present).flatMap(Collection::stream).distinct().collect(Collectors.toList());


        return neoPushDTOS;
    }

    public void getDisBySymptom(List<String> symptoms){
        List<String> allDis = new ArrayList<>();
        if(ListUtil.isNotEmpty(symptoms)){
            symptoms.forEach(x ->{
                List<Symptom> byNameIs = symptomNameRepository.findByNameIs(x);
                if(ListUtil.isNotEmpty(byNameIs)){
                    List<String> diseases = byNameIs.stream().filter(z -> z.getDisease() != null && z.getDisease().size() > 0)
                            .map(z -> z.getDisease().stream().map(y -> y.getName()).collect(Collectors.toList())).flatMap(List::stream)
                            .collect(Collectors.toList());
                    allDis.addAll(diseases);
                }
            });
        }

        Map<String, Long> dis_count = allDis.stream().collect(
                Collectors.groupingBy(Function.identity(), Collectors.counting())
        );
        if(dis_count != null && dis_count.size() > 0){
            dis_count = dis_count.entrySet().stream().sorted((o1,o2)->o2.getValue().compareTo(o1.getValue()))
                    .map(entry -> {
                        Map<String, Long> result = new LinkedHashMap();
                        result.put(entry.getKey(),entry.getValue());
                        return result;
                    }).reduce((map1,map2) ->{
                        map2.entrySet().forEach(entry ->map1.put(entry.getKey(),entry.getValue()));
                        return map1;
                    }).get();
            System.out.println();

//            String dis_first = dis_count.entrySet().stream().map(x -> x.getKey()).collect(Collectors.toList()).get(0);

        }



    }

    /**
     * 处理开单合理性图谱数据
     *
     * @param billNeoVO
     * @return
     */
    public List<BillNeoDTO> getBillNeo(BillNeoVO billNeoVO) {

        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();

        try {
            billNeoDTOs.addAll(getDrugBill(billNeoVO.getDrugList()));
            billNeoDTOs.addAll(getPacsBill(billNeoVO.getPacsList()));
            billNeoDTOs.addAll(getLisDetailBill(billNeoVO.getLisDetailList()));
            billNeoDTOs.addAll(getLisBill(billNeoVO.getLisList()));
            billNeoDTOs.addAll(getOperationBill(billNeoVO.getOperationList()));
            billNeoDTOs.addAll(getCombiOpBill(billNeoVO.getCombiopList()));
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }

        return billNeoDTOs;
    }


    /**
     * 处理高危项目图谱数据
     *
     * @param highRiskNeoVO
     * @return
     */
    public List<HighRiskNeoDTO> getHighRiskNeo(HighRiskNeoVO highRiskNeoVO) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();

        highRiskNeoDTOS.addAll(getHighRiskDrugList(highRiskNeoVO.getDrugList()));
        highRiskNeoDTOS.addAll(getHighRiskOperationList(highRiskNeoVO.getOperationList()));
        return highRiskNeoDTOS;
    }


    /**
     * 获取危急值图谱数据
     */
    public List<CriticalNeoDTO> getCritialNeo(CriticalNeoVO criticalNeoVO) {
        List<CriticalNeoDTO> criticalNeoDTOs = new ArrayList<>();

        criticalNeoDTOs.addAll(getLisCritical(criticalNeoVO));

        return criticalNeoDTOs;
    }


    /**
     * 处理处方开单合理性
     */
    public List<BillNeoDTO> getDrugBill(Map<String, String> medsterm) {

        MedicineCodeNode medicineNode = new MedicineCodeNode();
        List<BillNeoDTO> billNeoDTOs = medicineNode.getDrugBill(medsterm, medicineCodeRepository);

        return billNeoDTOs;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm) {

        PacsNameNode pacsNode = new PacsNameNode();
        List<BillNeoDTO> billNeoDTOs = pacsNode.getPacsBill(pacsterm, pacsNameRepository);

        return billNeoDTOs;
    }


    /**
     * 处理化验套餐开单合理性
     */
    public List<BillNeoDTO> getLisBill(Map<String, String> lissetterm) {

        LisSetNode lisSetNode = new LisSetNode();
        List<BillNeoDTO> billNeoDTOs = lisSetNode.getLisBill(lissetterm, lisSetRepository);

        return billNeoDTOs;
    }


    /**
     * 处理化验开单合理性
     */
    public List<BillNeoDTO> getLisDetailBill(Map<String, String> listerm) {

        LisNameNode lisNode = new LisNameNode();
        List<BillNeoDTO> billNeoDTOs = lisNode.getLisDetailBill(listerm, lisNameRepository);

        return billNeoDTOs;
    }



    /**
     * 处理手术开单合理性
     */
    public List<BillNeoDTO> getOperationBill(Map<String, String> listerm) {

        YiBaoOperationNameNode yiBaoOpNode = new YiBaoOperationNameNode();
        List<BillNeoDTO> billNeoDTOs = yiBaoOpNode.getOperationBill(listerm, yiBaoOperationRepository);

        return billNeoDTOs;
    }


    /**
     * 处理联合项目开单合理性
     */
    public List<BillNeoDTO> getCombiOpBill(Map<String, String> combiterm) {

        CombiOperationNode combiNode = new CombiOperationNode();
        List<BillNeoDTO> billNeoDTOS = combiNode.getCombiOpBill(combiterm, combiOperationRepository);

        return billNeoDTOS;
    }


    /**
     * 推送疾病信息
     *
     * @param neoPushVO
     * @return
     */
    public List<NeoPushDTO> getDiagInfo(NeoPushVO neoPushVO) {

        List<NeoPushDTO> neoPushDTOs = new ArrayList<>();
        NeoPushDTO pushDTO;

        YiBaoDiseaseNode icdDiseaseNode = new YiBaoDiseaseNode();
        String term;

        for (Diag diag : neoPushVO.getDiagVo().getDiags()) {
            term = diag.getName();
            List<YiBaoDiseaseName> yiBaoName = yiBaoDiseaseRepository.findByNameIs(term);
            YiBaoDiseaseName icdDisease = null;
            if(ListUtil.isNotEmpty(yiBaoName)){
                icdDisease = yiBaoName.get(0);
            }
            if (icdDisease != null) {
                pushDTO = icdDiseaseNode.YiBaoDiseasetoDiseaseDTO(icdDisease, neoPushVO);

                if (null != pushDTO) {
                    neoPushDTOs.add(pushDTO);
                }
            }
        }

        return neoPushDTOs;
    }


    /**
     * 获取化验危急值
     */
    public List<CriticalNeoDTO> getLisCritical(CriticalNeoVO criticalNeoVO) {

        LisNameNode lisNode = new LisNameNode();
        List<CriticalNeoDTO> criticalNeoDTOs = lisNode.getLisCritialList(criticalNeoVO, lisNameRepository);

        return criticalNeoDTOs;
    }

    /**
     * 处理高危药品数据
     *
     * @param drugs
     * @return
     */
    public List<HighRiskNeoDTO> getHighRiskDrugList(List<Drug> drugs) {
        MedRegNameNode medRegNameNode = new MedRegNameNode();
        List<HighRiskNeoDTO> highRiskNeoDTOS = medRegNameNode.getHighRiskDrug(drugs, medicineRepository, medicineCodeRepository);

        return highRiskNeoDTOS;
    }


    /**
     * 处理高危手术和操作数据
     *
     * @param operations
     * @return
     */
    public List<HighRiskNeoDTO> getHighRiskOperationList(Map<String, String> operations) {
        YiBaoOperationNameNode yiBaoOperationNode = new YiBaoOperationNameNode();
        List<HighRiskNeoDTO> highRiskNeoDTOS = yiBaoOperationNode.getHighRiskOperation(operations, yiBaoOperationRepository);

        return highRiskNeoDTOS;
    }

    /**
     * 提取方法
     *
     * @param jsonArray
     * @param originName
     * @param map
     */
    public void putMap(JSONArray jsonArray, String originName, Map<String, String> map) {
        if (jsonArray.size() == 2) {
            double likeRate = jsonArray.getDoubleValue(1);
            // System.out.println(originName + " " + likeRate);
            if (likeRate > 0.9) {
                String standName = jsonArray.getString(0);
                map.put(originName, standName);
            } else {
                map.put(originName, originName);
            }
        }
    }


    /**
     * 标准词转换
     * 类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 辅助检查：vital"
     *
     * @param standConvert
     * @return Map<String   ,       Map   <   String   ,       String>> -->Map<类型, Map<原始词, 标准词>>
     */
    public Map<String, Map<String, String>> standConvertCrf(StandConvert standConvert) {
        Map<String, Map<String, String>> map = new LinkedHashMap<>();

        List<StandConvertCrfVO> standConvertCrfVOList = new ArrayList<>();
        List<String> clinicalConList = getConvertList(standConvert.getClinicalList(), StandConvertEnum.symptom.getName(), standConvertCrfVOList);
        List<String> operationConList = getConvertList(standConvert.getOperationList(), StandConvertEnum.operation.getName(), standConvertCrfVOList);
        List<String> drugConList = getConvertList(standConvert.getDrugList(), StandConvertEnum.drug.getName(), standConvertCrfVOList);
        List<String> vitallConList = getConvertList(standConvert.getVitalList(), StandConvertEnum.vital.getName(), standConvertCrfVOList);
        List<String> diseaseConList = getConvertList(standConvert.getDiaglList(), StandConvertEnum.disease.getName(), standConvertCrfVOList);
        List<String> pacsConList = getConvertList(standConvert.getPacsList(), StandConvertEnum.pacs.getName(), standConvertCrfVOList);
        List<String> lisConList = getConvertList(standConvert.getLisList(), StandConvertEnum.lis.getName(), standConvertCrfVOList);
        List<String> transfusionConList = getConvertList(standConvert.getTransfusionList(), StandConvertEnum.transfusion.getName(), standConvertCrfVOList);

        StandConvertCrfBatchDTO standConvertCrfBatchDTO = null;
        try {
            standConvertCrfBatchDTO = standConvertServiceClient.similarityBatch(standConvertCrfVOList);
        } catch (Exception e) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "标准词转换【服务器】挂了!" + e.getMessage());
        }
        Map<String, Map<String, StandConvertCrfDTO>> crfMap = standConvertCrfBatchDTO.getData();
        getConvertMap(crfMap, StandConvertEnum.symptom.getName(), clinicalConList, standConvert.getClinicalList(), map);
        getConvertMap(crfMap, StandConvertEnum.operation.getName(), operationConList, standConvert.getOperationList(), map);
        getConvertMap(crfMap, StandConvertEnum.drug.getName(), drugConList, standConvert.getDrugList(), map);
        getConvertMap(crfMap, StandConvertEnum.vital.getName(), vitallConList, standConvert.getVitalList(), map);
        getConvertMap(crfMap, StandConvertEnum.disease.getName(), diseaseConList, standConvert.getDiaglList(), map);
        getConvertMap(crfMap, StandConvertEnum.pacs.getName(), pacsConList, standConvert.getPacsList(), map);
        getConvertMap(crfMap, StandConvertEnum.lis.getName(), lisConList, standConvert.getLisList(), map);
        getConvertMap(crfMap, StandConvertEnum.transfusion.getName(), transfusionConList, standConvert.getTransfusionList(), map);

        return map;
    }

    /**
     * 入参数据处理
     *
     * @param wordList
     * @param type
     * @param standConvertCrfVOList
     */
    public List<String> getConvertList(List<String> wordList, String type, List<StandConvertCrfVO> standConvertCrfVOList) {
        List<String> convertList = new ArrayList<>();
        if (ListUtil.isEmpty(wordList)) {
            return null;
        }

        // 从redis获取对应的内容
        Map<String, String> redisMap = redisUtil.getValueByType(type);
        if (redisMap == null) {
            redisMap = new LinkedHashMap<>();
        }
        if (ListUtil.isNotEmpty(wordList)) {
            for (String s : wordList) {
                if (StringUtil.isBlank(redisMap.get(s))) {
                    StandConvertCrfVO standConvertCrfVO = new StandConvertCrfVO();
                    standConvertCrfVO.setWord_type(type);
                    standConvertCrfVO.setWord(s);
                    standConvertCrfVOList.add(standConvertCrfVO);
                    if (!convertList.contains(s)) {
                        convertList.add(s);
                    }
                }
            }
        }
        return convertList;
    }

    /**
     * 获取标准词转换map结果，并更新redis
     *
     * @param crfDTO
     * @param type
     * @param convertList
     * @param orginList
     * @param map
     */
    public void getConvertMap(Map<String, Map<String, StandConvertCrfDTO>> crfDTO, String type, List<String> convertList,
                              List<String> orginList, Map<String, Map<String, String>> map) {
        if (ListUtil.isEmpty(orginList)) {
            return ;
        }
        Map<String, StandConvertCrfDTO> crfMap = crfDTO.get(type);
        Map<String, String> typeMap = new LinkedHashMap<>();
        Boolean updateRedis = false;

        // 从redis获取对应的内容
        Map<String, String> redisMap = redisUtil.getValueByType(type);
        if (redisMap == null) {
            redisMap = new LinkedHashMap<>();
        }
        for (String s : orginList) {
            if (redisMap.containsKey(s)) {
                typeMap.put(s, redisMap.get(s));
            } else if (crfMap != null) {
                StandConvertCrfDTO standConvertCrfDTO = crfMap.get(s);
                if (standConvertCrfDTO != null) {
                    List<Map<String,String>> list = standConvertCrfDTO.getStandard_words();
                    if (ListUtil.isEmpty(list)) {
                        redisMap.put(s, s);
                        typeMap.put(s, s);
                    } else {
                        String rateStr = standConvertCrfDTO.getStandard_words().get(0).get("rate");
                        BigDecimal rate = new BigDecimal(rateStr);
                        int flag = rate.compareTo(new BigDecimal(standConvertRate));
                        if (flag < 0) {
                            redisMap.put(s, s); // rate 小于阈值
                            typeMap.put(s, s);
                        } else {
                            redisMap.put(s, standConvertCrfDTO.getStandard_words().get(0).get("standard_word"));
                            typeMap.put(s, standConvertCrfDTO.getStandard_words().get(0).get("standard_word"));
                        }
                    }
                } else {
                    redisMap.put(s, s); // 找不到就认为是标准词
                    typeMap.put(s, s); // 找不到就认为是标准词
                }
                updateRedis = true;
            } else {
                redisMap.put(s, s); // 找不到就认为是标准词
                typeMap.put(s, s); // 找不到就认为是标准词
                updateRedis = true;
            }
        }

        // 更新redis
        if (updateRedis) {
            redisUtil.updateValueByType(redisMap, type);
        }
        map.put(type, typeMap);
    }
}
