package com.diagbot.process;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.IndicationDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.dto.OtherTipNeoDTO;
import com.diagbot.dto.WordCrfDTO;
import com.diagbot.enums.TypeEnum;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.StringUtil;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description: 其他提示总入口
 * @author: zhoutg
 * @time: 2018/8/6 9:11
 */
@Component
public class OtherTipProcess {

    /**
     * 处理业务
     *
     * @param otherTipNeoDTOList
     * @param wordCrfDTO
     * @param res
     */
    public void processLis(List<OtherTipNeoDTO> otherTipNeoDTOList, WordCrfDTO wordCrfDTO, IndicationDTO res) {
        // 化验结果与诊断和开单药品进行匹配
        for (OtherTipNeoDTO otherTipNeoDTO : otherTipNeoDTOList) {
            List<NodeNeoDTO> factor = otherTipNeoDTO.getFactor();

            if (ListUtil.isNotEmpty(factor)) {
                for (NodeNeoDTO nodeNeoDTO : factor) {
                    Boolean flag = false;
                    if (TypeEnum.disease.getName().equals(nodeNeoDTO.getTermtype())) { // 诊断
                        flag = CoreUtil.compareNameWithNodeNeoDTO(nodeNeoDTO, wordCrfDTO.getDiag());
                    } else if (TypeEnum.drug.getName().equals(nodeNeoDTO.getTermtype())) { // 药品
                        flag = CoreUtil.compareNameWithNodeNeoDTO(nodeNeoDTO, wordCrfDTO.getDrugOrder());
                    }
                    if (flag) {
                        String content = otherTipNeoDTO.getName() + otherTipNeoDTO.getDetailName();
                        if (StringUtil.isNotBlank(otherTipNeoDTO.getOtherValue())) {
                            content += otherTipNeoDTO.getOtherValue();
                        } else {
                            content += otherTipNeoDTO.getValue();
                        }
                        BillMsg billMsg = CoreUtil.getCommonOtherTipLisMsg(content, nodeNeoDTO.getName(), nodeNeoDTO.getTermtype());
                        res.getOtherList().add(billMsg);
                    }
                }
            }
        }
    }
}
