package com.diagbot.repository;

import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.PacsBillNeoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.util.NeoUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class PacsNameNode {


    public PacsBillNeoDTO PacstoPACSDTO(PacsName pacs) {
        PacsBillNeoDTO pacsBillNeoDTO = new PacsBillNeoDTO();
        NodeInfo nodeInfo;


        Gender gender = pacs.getGender();
        if (gender!=null) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(gender.getName());
            nodeInfo.setTypeval(Constants.gender);
            pacsBillNeoDTO.setGender(NeoUtil.updateNodeInfo(nodeInfo));
        }



        Set<ClinicalFinding> clinicalfindings = pacs.getClinicalfindings();
        for (ClinicalFinding clinicalfinding : clinicalfindings) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(clinicalfinding.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval()));
            nodeInfo.setUnit(clinicalfinding.getUnit());
            nodeInfo.setRange(clinicalfinding.getRange());
            nodeInfo.setTypeval(Constants.clinicalfinding);
            pacsBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Vital> vitals = pacs.getVitals();
        for (Vital vital : vitals) {
            nodeInfo = new NodeInfo();
            String name = vital.getName();
            nodeInfo.setName(name);
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(vital.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(vital.getMaxval()));
            nodeInfo.setUnit(vital.getUnit());
            nodeInfo.setRange(vital.getRange());
            nodeInfo.setTypeval(Constants.vital);
            pacsBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Lis> liss = pacs.getLis();
        for (Lis lis : liss) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(lis.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(lis.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(lis.getMaxval()));
            nodeInfo.setUnit(lis.getUnit());
            nodeInfo.setRange(lis.getRange());
            nodeInfo.setTypeval(Constants.lis);
            pacsBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<PacsDescribe> conflict_pacsdesc = pacs.getPacsdescribes();
        for (PacsDescribe pacsdesc : conflict_pacsdesc) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(pacsdesc.getName());
            nodeInfo.setTypeval(Constants.pacsdesc);
            pacsBillNeoDTO.getPacsDesc().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Group> groups = pacs.getGroups();
        for (Group group : groups) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(group.getName());
            nodeInfo.setTypeval(Constants.group);
            pacsBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<OralMedicine> conflict_medicines = pacs.getOralmedicines();
        for (OralMedicine conflict_medicine : conflict_medicines) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflict_medicine.getName());
            nodeInfo.setTypeval(Constants.oralmed);
            pacsBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<MedAllergen> medAllergens = pacs.getMedallergens();
        for (MedAllergen medAllergen : medAllergens) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(medAllergen.getName());
            nodeInfo.setTypeval(Constants.medallergen);
            pacsBillNeoDTO.getAllergicmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Disease> diseases = pacs.getDiseases();
        for (Disease disease : diseases) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(disease.getName());
            nodeInfo.setTypeval(Constants.disease);
            pacsBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<Allergen> allergens = pacs.getAllergens();
        for (Allergen allergen : allergens) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(allergen.getName());
            nodeInfo.setTypeval(Constants.allergen);
            pacsBillNeoDTO.getOperations().add(NeoUtil.updateNodeInfo(nodeInfo));
        }


        Set<ConflictDevice> conflictDevices = pacs.getConflictdevices();
        for (ConflictDevice conflictDevice : conflictDevices) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflictDevice.getName());
            nodeInfo.setTypeval(Constants.conflictdevice);
            pacsBillNeoDTO.getProcedures().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        return pacsBillNeoDTO;
    }


    /**
     * 处理辅检开单合理性
     */
    public List<BillNeoDTO> getPacsBill(Map<String, String> pacsterm, PacsNameRepository pacsRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        PacsBillNeoDTO pacsBillNeoDTO;

        List<PacsName> pacslt;
        String pacsname;

        for (String term : pacsterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            pacsname = pacsterm.get(term);
            billNeoDTO.setName(term);
            billNeoDTO.setStandname(pacsname);

            pacsBillNeoDTO = new PacsBillNeoDTO();

            pacslt = pacsRepository.findByNameIs(pacsname);

            if (pacslt != null && pacslt.size()>0) {
                pacsBillNeoDTO = PacstoPACSDTO(pacslt.get(0));
            }

            billNeoDTO.setPacsBillNeoDTO(pacsBillNeoDTO);

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }


    /**
     * 获取辅检名称列表
     */
    public List<String> getPACSNameList(String name, PacsNameRepository pacsRepository) {
        String pacsname;

        List<String> pacslist = new ArrayList<>();

        List<PacsName> pacslt = pacsRepository.findByNameContaining(name);

        for (PacsName pacs : pacslt) {
            pacsname = pacs.getName();
            if (!pacslist.contains(pacsname)) {
                pacslist.add(pacsname);
            }
        }


        return pacslist;
    }
}






