package com.diagbot.repository;


import com.diagbot.dto.BillNeoDTO;
import com.diagbot.dto.HighRiskNeoDTO;
import com.diagbot.dto.OperationBillNeoDTO;
import com.diagbot.dto.OperationInfoDTO;
import com.diagbot.entity.node.*;
import com.diagbot.util.ListUtil;
import com.diagbot.util.NeoUtil;

import javax.xml.soap.Node;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

public class YiBaoOperationNameNode {


    public OperationBillNeoDTO YiBaoOperationtoOperationDTO(YiBaoOperationName operation) {
        OperationBillNeoDTO opBillNeoDTO = new OperationBillNeoDTO();
        NodeInfo nodeInfo;

        Gender gender = operation.getGender();
        if (null!=gender) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(gender.getName());
            nodeInfo.setTypeval(Constants.gender);
            opBillNeoDTO.setGender(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<ClinicalFinding> clinicalfindings = operation.getClinicalFindings();
        for (ClinicalFinding clinicalfinding : clinicalfindings) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(clinicalfinding.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(clinicalfinding.getMaxval()));
            nodeInfo.setUnit(clinicalfinding.getUnit());
            nodeInfo.setRange(clinicalfinding.getRange());
            nodeInfo.setTypeval(Constants.clinicalfinding);
            opBillNeoDTO.getClinicfindings().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Vital> vitals = operation.getVitals();
        for (Vital vital : vitals) {
            nodeInfo = new NodeInfo();
            String name = vital.getName();
            nodeInfo.setName(name);
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(vital.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(vital.getMaxval()));
            nodeInfo.setUnit(vital.getUnit());
            nodeInfo.setRange(vital.getRange());
            nodeInfo.setTypeval(Constants.vital);
            opBillNeoDTO.getVitals().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Lis> lisSet = operation.getLiss();
        for (Lis lis : lisSet) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(lis.getName());
            nodeInfo.setMinval(NeoUtil.DoubletoBigDecimal(lis.getMinval()));
            nodeInfo.setMaxval(NeoUtil.DoubletoBigDecimal(lis.getMaxval()));
            nodeInfo.setUnit(lis.getUnit());
            nodeInfo.setRange(lis.getRange());
            nodeInfo.setTypeval(Constants.lis);
            opBillNeoDTO.getLis().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<OralMedicine> conflict_oralmeds = operation.getOralmeds();
        for (OralMedicine conflict_oralmed : conflict_oralmeds) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(conflict_oralmed.getName());
            nodeInfo.setTypeval(Constants.oralmed);
            opBillNeoDTO.getOralmeds().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Disease> conflict_disease = operation.getDiseases();
        for (Disease disease : conflict_disease) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(disease.getName());
            nodeInfo.setTypeval(Constants.disease);
            opBillNeoDTO.getDisease().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<Group> conflict_group = operation.getGroups();
        for (Group group : conflict_group) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(group.getName());
            nodeInfo.setTypeval(Constants.group);
            opBillNeoDTO.getGroup().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<PacsDescribe> conflict_pacsdesc = operation.getPacsdescs();
        for (PacsDescribe pacsdesc : conflict_pacsdesc) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(pacsdesc.getName());
            nodeInfo.setTypeval(Constants.pacsdesc);
            opBillNeoDTO.getPacsDesc().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<MedAllergen> conflict_medallegen = operation.getMedallergens();
        for (MedAllergen medAllergen : conflict_medallegen) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(medAllergen.getName());
            nodeInfo.setTypeval(Constants.medallergen);
            opBillNeoDTO.getMedallegen().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        Set<ConflictDevice> conflictDevices = operation.getConflictdevices();
        for (ConflictDevice device : conflictDevices) {
            nodeInfo = new NodeInfo();
            nodeInfo.setName(device.getName());
            nodeInfo.setTypeval(Constants.conflictdevice);
            opBillNeoDTO.getMedallegen().add(NeoUtil.updateNodeInfo(nodeInfo));
        }

        return opBillNeoDTO;
    }

    /**
     * 处理手术开单合理性
     */
    public List<BillNeoDTO> getOperationBill(Map<String, String> opterm, YiBaoOperationNameRepository opRepository) {
        List<BillNeoDTO> billNeoDTOs = new ArrayList<>();
        BillNeoDTO billNeoDTO;

        OperationBillNeoDTO opBillNeoDTO;

        String opname;

        for (String term : opterm.keySet()) {
            billNeoDTO = new BillNeoDTO();
            opname = opterm.get(term);
            billNeoDTO.setName(opname);

            List<YiBaoOperationName> operations = opRepository.findByNameIs(opname);

            if (null!=operations && operations.size()>0) {
                opBillNeoDTO = YiBaoOperationtoOperationDTO(operations.get(0));
                billNeoDTO.setOperationBillNeoDTO(opBillNeoDTO);
            }

            billNeoDTOs.add(billNeoDTO);
        }

        return billNeoDTOs;
    }

    /**
     * 获取手术操作名称列表
     */
    public List<OperationInfoDTO> getOperationList(String name, YiBaoOperationNameRepository operationRepository) {
        String nodename;

        List<OperationInfoDTO> opslist = new ArrayList<>();
        OperationInfoDTO opInfoDTO;

        List<YiBaoOperationName> operations = operationRepository.findByNameContaining(name);

        for (YiBaoOperationName op : operations) {
            opInfoDTO = new OperationInfoDTO();
            nodename = op.getName();

            opInfoDTO.setName(nodename);
            opslist.add(opInfoDTO);
        }


        return opslist;
    }



    /**
     * 获取高危手术和操作数据
     */
    public List<HighRiskNeoDTO> getHighRiskOperation(Map<String, String> opsterm, YiBaoOperationNameRepository operationRepository) {
        List<HighRiskNeoDTO> highRiskNeoDTOS = new ArrayList<>();
        HighRiskNeoDTO highRiskNeoDTO;
        String opname, opgrade;
        YiBaoOperationName operation;

        for (String term : opsterm.keySet()) {
            highRiskNeoDTO = new HighRiskNeoDTO();

            opname = opsterm.get(term).trim();
            highRiskNeoDTO.setName(term);
            highRiskNeoDTO.setStandname(opname);

            List<YiBaoOperationName> operations = operationRepository.findByNameIs(opname);

            if (ListUtil.isNotEmpty(operations)) {
                operation = operations.get(0);
                if (null!=operation.getOpgrade() && operation.getOpgrade().trim().length()>0) {
                    opgrade = operation.getOpgrade().trim();
                    if (opgrade.equals("3") || opgrade.equals("4")) {
                        highRiskNeoDTO.setIsHighrisk(true);
                        highRiskNeoDTO.setType(Constants.shoushu);
                        highRiskNeoDTOS.add(highRiskNeoDTO);
                    }
                }
            }
        }

        return highRiskNeoDTOS;
    }

}
