package com.diagbot.rule;

import com.diagbot.dto.BillMsg;
import com.diagbot.dto.BillNeoMaxDTO;
import com.diagbot.dto.NodeNeoDTO;
import com.diagbot.model.entity.Negative;
import com.diagbot.util.CoreUtil;
import com.diagbot.util.ListUtil;
import org.apache.commons.lang3.StringUtils;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @description: 实验室检查和辅助检查规则
 * @author: kwz
 * @time: 2020/8/3 14:47
 */
public class DrugRule {

    /**
     * 比较药品是否存在(图谱返回药品通用名或者药品分类，如果返回的是药品分类，再从redis获取药品通用名)
     *
     * @param drugList
     * @param inputList
     * @return
     */
    public static <T> void compareDrugWithBill(List<NodeNeoDTO> drugList, List<T> inputList, BillNeoMaxDTO billNeoMaxDTO,
                                               List<BillMsg> billMsgList, Map<String, List<String>> drugMap, String type) {
        Map<String, String> old_stand = new HashMap<>();
        if (ListUtil.isNotEmpty(drugList) && ListUtil.isNotEmpty(inputList)) {
            List<String> allDrug = new ArrayList<>();
            for (NodeNeoDTO nodeNeoDTO : drugList) {
                if ("服用药品".equals(nodeNeoDTO.getTermtype())) {
                    allDrug.add(nodeNeoDTO.getName());
                } else if ("药品解剖学类别".equals(nodeNeoDTO.getTermtype())){
                    String drugType = nodeNeoDTO.getName();
                    if (drugMap.get(drugType) != null) {
                        allDrug.addAll(drugMap.get(drugType));
                    }
                }
            }
            for (T d : inputList) {
                Negative val = (Negative) CoreUtil.getFieldValue(d, "negative");
                if (val == null) {
                    String standName = (String) CoreUtil.getFieldValue(d, "standName");
                    String name = (String) CoreUtil.getFieldValue(d, "name");
                    if (StringUtils.isNotBlank(standName) && allDrug.contains(standName)) {
                        old_stand.put(name, standName);
                    }
                }
            }
            if (old_stand.size() > 0) {
                for (Map.Entry<String, String> d : old_stand.entrySet()) {
                    String message = "";
                    if ("禁忌过敏药品".equals(type)) {
                        message = d.getKey() + "过敏";
                    } else {
                        message = "已开" + d.getKey();
                    }
                    BillMsg commonBillMsg = CoreUtil.getCommonBillMsg(
                            billNeoMaxDTO.getOrderName(), billNeoMaxDTO.getOrderStandName(),
                            message, type);
                    billMsgList.add(commonBillMsg);
                }
            }
        }
    }
}
