package com.diagbot.web;

import com.diagbot.dto.RespDTO;
import com.diagbot.dto.StandConvertCrfBatchDTO;
import com.diagbot.dto.StandConvertCrfDTO;
import com.diagbot.facade.TestFacade;
import com.diagbot.vo.StandConvertCrfVO;
import com.diagbot.vo.TestLineVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;
import java.util.Map;

/**
 * @Description: CDSS核心测试API控制层
 * @author: zhoutg
 * @time: 2018/8/30 10:12
 */
@RestController
@RequestMapping("/test")
@Api(value = "测试API", tags = { "测试API" })
@SuppressWarnings("unchecked")
public class TestController {

    @Autowired
    private TestFacade testFacade;

    @ApiOperation(value = "开单合理性测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testIndication")
    public RespDTO<Map<String, Object>> testIndication(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.importExcel(file, testLineVO));
    }

    @ApiOperation(value = "危急值测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testCritical")
    public RespDTO<Map<String, Object>> testCritical(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.importCriticalExcel(file, testLineVO));
    }

    @ApiOperation(value = "高危手术测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testHighRiskOperation")
    public RespDTO<Map<String, Object>> testHighRiskOperation(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testHighRiskOperationExcel(file, testLineVO));
    }

    @ApiOperation(value = "高危药品测试API[zhoutg]",
            notes = "idNum：指定行测试<br>" +
                    "sheetIndex：sheet的下标，默认为0，表示第一个sheet")
    @PostMapping("/testHighRiskDrug")
    public RespDTO<Map<String, Object>> testHighRiskDrug(@RequestParam("file") MultipartFile file, TestLineVO testLineVO) {
        return RespDTO.onSuc(testFacade.testHighRiskDrugExcel(file, testLineVO));
    }

    @ApiOperation(value = "标准词转换API[zhoutg]", notes = "类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 辅助检查：vital")
    @PostMapping("/testStandConvert")
    public RespDTO<StandConvertCrfDTO> testStandConvert(@RequestBody StandConvertCrfVO standConvertCrfVO) {
        return RespDTO.onSuc(testFacade.testStandConvert(standConvertCrfVO));
    }

    @ApiOperation(value = "标准词批量转换API[zhoutg]", notes = "类型，疾病: disease，症状: symptom，手术和操作:operation，药品: drug，实验室检查：lis，辅助检查：pacs， 辅助检查：vital")
    @PostMapping("/testStandConvertBatch")
    public RespDTO<StandConvertCrfBatchDTO> testStandConvertBatch(@RequestBody List<StandConvertCrfVO> standConvertCrfVOList) {
        return RespDTO.onSuc(testFacade.testStandConvertBatch(standConvertCrfVOList));
    }
}

